require "google/ads/google_ads/version_alternate"

require "google/ads/google_ads/factories/v7/resources"
require "google/ads/google_ads/factories/v7/services"
require "google/ads/google_ads/factories/v7/enums"
require "google/ads/google_ads/factories/v7/operations"

require "google/ads/google_ads/factories/v8/resources"
require "google/ads/google_ads/factories/v8/services"
require "google/ads/google_ads/factories/v8/enums"
require "google/ads/google_ads/factories/v8/operations"

module Google
  module Ads
    module GoogleAds
      module Factories
        Factory = Struct.new(:resources, :services, :enums, :operations)

        FACTORY_V7 = Factory.new(
          V7::Resources,
          V7::Services,
          V7::Enums,
          V7::Operations
        ).freeze

        FACTORY_V8 = Factory.new(
          V8::Resources,
          V8::Services,
          V8::Enums,
          V8::Operations
        ).freeze

        VERSIONS = [

          :V7,

          :V8

        ]

        HIGHEST_VERSION = :V8

        def self.version_alternate_for(type)
          unless [:resources, :services, :enums, :operations].include?(type)
            raise ArgumentError.new(
              "Dont have version alternate for #{type}, valid values are :resources, :services, :enums, :operations,  got #{type}"
            )
          end

          VersionAlternate.new(
            FACTORY_V8.public_send(type),
            {

              V7: FACTORY_V7.public_send(type),

              V8: FACTORY_V8.public_send(type)

            }
          )
        end

        def self.versions
        end

        def self.at_version(version)
          case version

          when :V7
            FACTORY_V7

          when :V8
            FACTORY_V8

          else
            raise ArgumentError.new("Got unkown version: #{version}")
          end
        end
      end
    end
  end
end
